package Uebung07;

import java.util.ArrayList;
import java.util.Scanner;

/**
 * <h1>Übung 07, Aufgabe 01</h1>
 *
 * <p>Programmieren 1
 * Wintersemester 2021/-22
 * HS-Bremerhaven
 * Prof. Dr. Peter Kelb</p>
 *
 * <p>Verändern Sie die Initialisierung der Zellen in Ihrem Life Programm. Die Zellen
 * sollen jetzt zufällig mit booleschen Werten initialisiert werden.</p>
 * <p>Verwenden Sie dazu den Methodenaufruf java.lang.Math.random(). Dieser
 * liefert einen zufälligen Double-Wert zwischen 0.0 und 1.0 zurück.</p>
 * <p>Schreiben Sie eine Methode, die einen int-Wert x übergeben bekommt und alle
 * Zellen ermittelt, die genau x Nachbarn haben. Die Koordinaten der Zellen sollen
 * als Array zurückgeliefert werden.</p>
 * <p>Programmieren Sie die vorherige Methode erneut, so dass sie rekursiv arbeitet.</p>
 *
 * @author franjogiessel
 */
public class Aufgabe01 {

    public static void main(String[] args) {
        Scanner sc = new Scanner(System.in); // neues Scanner-Objekt sc

        System.out.print("Anzahl Zeilen: ");
        int zeilen = sc.nextInt();
        System.out.print("Anzahl Spalten: ");
        int spalten = sc.nextInt();

        boolean[][] ausgabe = initializeArray(zeilen, spalten);
        printArray(ausgabe);

        for (int i = 0; i < spalten; i++) {
            System.out.print('-');
        }
        System.out.println();

        boolean[][] neueAusgabe = recalculateArray(ausgabe);
        printArray(neueAusgabe);
    }

    /**
     * Initialisiert ein zweidimensionales-boolean-Array nach den gegebenen Vorgaben.
     * Die Größe wird duch die Parameter festgelegt.
     * @param zeilen Anzahl Zeilen des Arrays
     * @param spalten Anzahl Spalen des Arrays
     * @return zweidimensionales-boolean-Array nach den gegebenen Vorgaben.
     */
    public static boolean[][] initializeArray(int zeilen, int spalten) {
        boolean[][] ausgabe = new boolean[zeilen][spalten];

        // Array betanken
        for (int i = 0; i < ausgabe.length; ++i) {
            for (int j = 0; j < ausgabe[i].length; ++j) {
                ausgabe[i][j] = java.lang.Math.random() > 0.5 ? true : false;
            }
        }
        return ausgabe;
    }

    /**
     * Gibt das übergebene zweidimensionale-boolean-Array zeilenweise auf StOut aus.
     * @param ausgabe Auszugebenes zweidimensionales-boolean-Array
     */
    public static void printArray(boolean[][] ausgabe) {
        // Array ausgeben
        for (boolean[] zeile : ausgabe) { // Die Objekte von ausgabe[][] sind zeile[]
            for (boolean zelle : zeile) { // Die Objekte von zeile[] sind zelle
                System.out.print(zelle ? 'T' : 'F');
            }
            System.out.println();
        }
    }

    /**
     * Berechnet das neue zweidimensionale-boolean-Array nach den neuen Vorgaben.
     * @param ausgabe Ausgangs-zweidimensionales-boolean-Array
     * @return neues zweidimensionales-boolean-Array nach den neuen Voragaben.
     */
    public static boolean[][] recalculateArray(boolean[][] ausgabe) {
        // neues Ausgabe-Array erstellen mit derselben Größe, wie das Ausgangsarray
        boolean[][] neueAusgabe = new boolean[ausgabe.length][ausgabe[1].length];

        // Array betanken
        for (int i = 0; i < neueAusgabe.length; ++i) {
            for (int j = 0; j < neueAusgabe[i].length; ++j) {

                // Nachbarschaft berechnen
                int nachbarschaft = getNeighborhood(ausgabe, i, j);

                // neuen Wert entsprechend der vorgegebenen Regeln setzen
                neueAusgabe[i][j] = nachbarschaft == 3 || (nachbarschaft == 2 && ausgabe[i][j]);
            }
        }
        return neueAusgabe;
    }

    /**
     * Gibt die Koordinaten der Zellen von ausgabe aus, die x Nachbarn haben.
     * @param ausgabe Das Array, indem die Zellen gesucht werden sollen.
     * @param x Die Anzahl an Nachbarn, die die Zellen haben sollen.
     * @return Zweidimensionales-Int-Array, dass die Koordinaten der Zellen enthält.
     * Der erste Wert des zweiten Arrays ist die x-Koordinate, der zweite Wert die y-Koordinate.
     */
    public static int[][] getCellWithNeighborhoodX (boolean[][] ausgabe, int x) {
        ArrayList<int[]> cells = new ArrayList<int[]>();

        for (int i = 0; i < ausgabe.length; ++i) {
            for (int j = 0; j < ausgabe[i].length; ++j) {
                if (getNeighborhood(ausgabe, i, j) == x) {
                    cells.add(new int[] {i, j});
                }
            }
        }
        int[][] foundCells = new int[cells.size()][2];
        for (int i = 0; i < cells.size(); ++i) {
            foundCells[i][0] = cells.get(i)[0];
            foundCells[i][1] = cells.get(i)[1];
        }
        return foundCells;
    }

    public static int[][] getCellWithNeighborhoodXRekursiv (boolean[][] ausgabe, int x) {
        return getCellWithNeighborhoodXRekursiv(ausgabe, x, 0, 0, new int[0][2]);
    }

    private static int[][] getCellWithNeighborhoodXRekursiv (boolean[][] ausgabe, int neighborhood, int x, int y, int[][] bisherigeKoordinaten) {
        // bisheriges Array kopieren und um 1 vergrößern
        int[][] neueKoordinaten = new int[bisherigeKoordinaten.length + 1][2];
        for (int i = 0; i < bisherigeKoordinaten.length; ++i) {
            neueKoordinaten[i] = bisherigeKoordinaten[i];
        }

        if (getNeighborhood(ausgabe, y, x) == neighborhood) {
            neueKoordinaten[neueKoordinaten.length -1][0] = x;
            neueKoordinaten[neueKoordinaten.length -1][1] = y;
            return neueKoordinaten;
        } else {
            getCellWithNeighborhoodXRekursiv(ausgabe, neighborhood, x+1, y, bisherigeKoordinaten);
        }
        return neueKoordinaten;
    }

    /**
     * Berechnet die Nachbarschaft der durch zeile und spalte definierte Zelle nach den neuen Vorgaben.
     * @param ausgabe Ausgangs-zweidimensionales-boolean-Array, auf die die Berechnung der Nachbarschaft basiert.
     * @param zeile zeile der zu berechnenden Zelle
     * @param spalte spalte der zu berechnenden Zelle
     * @return berechnete Nachbarschaft nach den neuen Voragaben.
     */
    private static int getNeighborhood(boolean[][] ausgabe, int zeile, int spalte) {
        int nachbarschaft = 0;

        for (int i = -1; i < 2; ++i) {
            for (int j = -1; j < 2; ++j) {
                if ((i != 0 || j != 0) && (ausgabe[(zeile + i + ausgabe.length) % ausgabe.length][(spalte + j + ausgabe[zeile].length) % ausgabe[zeile].length])) {
                    ++nachbarschaft;
                }
            }
        }
        return nachbarschaft;
    }
}
