package Uebung10;

import java.util.Scanner;

class Life {
    int[][] board;

    public Life(int zeilen, int spalten) {
        this.board = new int[zeilen][spalten];
        initializeArray();
    }

    /**
     * Initialisiert ein zweidimensionales-int-Array.
     */
    private void initializeArray() {
        for (int i = 0; i < this.board.length; ++i) {
            for (int j = 0; j < this.board[i].length; ++j) {
                this.board[i][j] = Math.random() > 0.5 ? 1 : 0;
            }
        }
    }

    /**
     * Berechnet das zweidimensionale-int-Array nach den neuen Vorgaben.
     */
    public void recalculateArray() {
        // neues Ausgabe-Array erstellen mit derselben Größe, wie das Ausgangsarray
        int[][] newBoard = new int[this.board.length][this.board[1].length];

        // Array betanken
        for (int i = 0; i < newBoard.length; ++i) {
            for (int j = 0; j < newBoard[i].length; ++j) {

                // Nachbarschaft berechnen
                int nachbarschaft = getNeighborhood(i, j);

                // neuen Wert entsprechend der vorgegebenen Regeln setzen
                newBoard[i][j] = (nachbarschaft == 3 || (nachbarschaft == 2 && this.board[i][j] > 0)) ? this.board[i][j]+1 : 0;
            }
        }
        this.board = newBoard;
    }

    /**
     * Berechnet die Nachbarschaft der durch zeile und spalte definierte Zelle (Wenn Nachbarn > 0 wird dieser gezählt)
     * @param zeile zeile der zu berechnenden Zelle
     * @param spalte spalte der zu berechnenden Zelle
     * @return berechnete Nachbarschaft nach den neuen Vorgaben.
     */
    private int getNeighborhood(int zeile, int spalte) {
        int nachbarschaft = 0;

        for (int i = -1; i < 2; ++i) {
            for (int j = -1; j < 2; ++j) {
                if ((i != 0 || j != 0) && (this.board[(zeile + i + this.board.length) % this.board.length][(spalte + j + this.board[zeile].length) % this.board[zeile].length] > 0)) {
                    ++nachbarschaft;
                }
            }
        }
        return nachbarschaft;
    }

    /**
     * Gibt das übergebene zweidimensionale-boolean-Array zeilenweise auf StOut aus.
     */
    public void printArray() {
        // Array ausgeben
        for (int[] zeile : this.board) { // Die Objekte von ausgabe[][] sind zeile[]
            for (int zelle : zeile) { // Die Objekte von zeile[] sind zelle
                char c = '*';
                switch (zelle) {
                    case 0:
                        c = ' ';
                        break;
                    case 1:
                        c = '.';
                        break;
                    case 2:
                        c = 'o';
                        break;
                    case 3:
                        c = 'O';
                        break;
                }
                System.out.print(c);
            }
            System.out.println();
        }
    }
}

/**
 * <h1>Übung 10, Aufgabe 01</h1>
 *
 * <p>Programmieren 1
 * Wintersemester 2021/-22
 * HS-Bremerhaven
 * Prof. Dr. Peter Kelb</p>
 *
 * <p>Stellen Sie Ihr Life Programm auf Klassen Objekte um.</p>
 * <p>Die Methoden sollen Objektmethoden werden und bekommen keine booleschen
 * Arrays mehr übergeben, sondern greifen auf Objektvariablen zu.</p>
 * <p>Ersetzen Sie auch das boolesche Array durch ein int-Array.</p>
 * <p>Mit dem int-Array soll kodiert werden, ob eine Zelle besetzt ist, und wenn Sie
 * besetzt ist, wie lange sie schon besetzt ist.</p>
 * <p>Verändern Sie die Ausgabe wie folgt:</p>
 * <ul><li>Wert=0 → Ausgabe: Leerzeichen</li>
 * <li>Wert=1 → Ausgabe: ‘.‘</li>
 * <li>Wert=2 → Ausgabe: ‘o‘</li>
 * <li>Wert=3 → Ausgabe: ‘O‘</li>
 * <li>Wert>3 → Ausgabe: ‘*‘</li></ul>
 *
 * @author franjogiessel
 */
public class Aufgabe01 {
    public static void main(String[] args) {
        Scanner sc = new Scanner(System.in); // neues Scanner-Objekt sc

        System.out.print("Anzahl Zeilen: ");
        int zeilen = sc.nextInt();
        System.out.print("Anzahl Spalten: ");
        int spalten = sc.nextInt();

        Life life = new Life(zeilen, spalten);
        life.printArray();

        for (int i = 1; i <= 5; ++i) {
            System.out.println("------------------");
            System.out.println("Nach " + i + " Tag" + (i == 1 ? "" : "en" )+ ":");
            life.recalculateArray();
            life.printArray();
        }

    }
}

