package Uebung10;

class Bruch {
    private int zaehler;
    private int nenner;

    /**
     * Erstellt ein Objekt der Klasse Bruch.
     * @param zaehler Zähler-Wert des Bruchs.
     * @param nenner Nenner-Wert des Bruchs.
     */
    public Bruch (int zaehler, int nenner) {
        this.zaehler = nenner < 0 ? -zaehler : zaehler;
        this.nenner = nenner < 0 ? -nenner : nenner;

        kuerzen();
    }

    /**
     * Gibt den Zähler-Wert des Bruchs zurück.
     * @return Zähler-Wert des Bruchs.
     */
    public int getZaehler () {
        return this.zaehler;
    }

    /**
     * Gibt den Nenner-Wert des Bruchs zurück.
     * @return Nenner-Wert des Bruchs.
     */
    public int getNenner () {
        return this.nenner;
    }

    /**
     * Kürzt den Bruch.
     */
    public void kuerzen() {
        int ggt0 = ggt(Math.abs(zaehler), Math.abs(nenner));
        this.zaehler = zaehler / ggt0;
        this.nenner = nenner / ggt0;
    }

    /**
     * Gibt den Bruch auf der Console aus.
     */
    public void print() {
        System.out.println(this.zaehler + " / " + this.nenner);
    }

    /**
     * Berechnet den größten gemeinsamen Teiler (ggT) der Zahlen a und b und gibt diesen wieder zurück.
     * @param a Erste Zahl
     * @param b Zweite Zahl
     * @return größten gemeinsamen Teiler.
     */
    private static int ggt(int a, int b) {
        // Die größere der beiden Zahlen muss als erstes an den Algorithmus übergeben werden.
        int va = a > b ? a : b;
        int vb = a > b ? b : a;
        int r = 1, vr = 1;
        while (r != 0) {
            vr = r;
            r = va % vb;
            va = vb;
            vb = r;
        }
        return vr;
    }

    /**
     * Multipliziert den Bruch mit dem übergebenen Bruch.
     * @param bruch2 zweiter zu multiplizierender Bruch.
     * @return Ergebnis der Multiplikation als Bruch.
     */
    public Bruch multiplikation(Bruch bruch2) {
        return new Bruch(this.zaehler * bruch2.zaehler, this.nenner * bruch2.nenner);
    }

    /**
     * Dividiert den Ausgangsbruch durch den übergebenen Bruch.
     * @param bruch2 Divisor-Bruch.
     * @return Ergebnis der Division als Bruch.
     */
    public Bruch division(Bruch bruch2) {
        return new Bruch(this.zaehler * bruch2.nenner , this.nenner * bruch2.zaehler);
    }

    /**
     * Addiert den Bruch mit dem übergebenen Bruch.
     * @param bruch2 zweiter zu addierender Bruch.
     * @return Ergebnis der Addition als Bruch.
     */
    public Bruch addition(Bruch bruch2) {
        return new Bruch (this.zaehler * bruch2.nenner + bruch2.zaehler * this.nenner , this.nenner * bruch2.nenner);
    }

    /**
     * Subtrahiert den übergebenen Bruch von dem Ausgangsbruch.
     * @param bruch2 zu subtrahierender Bruch.
     * @return Ergebnis der Subtraktion als Bruch.
     */
    public Bruch subtraktion(Bruch bruch2) {
        return new Bruch (this.zaehler * bruch2.nenner - bruch2.zaehler * this.nenner , this.nenner * bruch2.nenner);
    }
}

public class Aufgabe02 {
    public static void main(String[] args) {
        Bruch bruch1 = new Bruch (-4, 6);
        Bruch bruch2 = new Bruch (15, 24);

        Bruch addierterBruch = bruch1.addition(bruch2);
        addierterBruch.print();
        bruch1.subtraktion(bruch2).print();
        bruch1.multiplikation(bruch2).print();
        bruch1.division(bruch2).print();
    }
}
