package Uebung11;

import java.util.Scanner;

/**
 * Repräsentiert ein Spielfeld des game of life.
 */
class Life {
    Stone[][] board;

    /**
     * Erstellt ein neues Life-Objekt.
     */
    public Life(int zeilen, int spalten) {
        this.board = new Stone[zeilen][spalten];
        initializeArray();
    }

    /**
     * Initialisiert ein zweidimensionales-int-Array.
     */
    private void initializeArray() {
        for (int i = 0; i < this.board.length; ++i) {
            for (int j = 0; j < this.board[i].length; ++j) {
                this.board[i][j] = new Stone(Math.random() > 0.5 ? 1 : 0);
            }
        }
    }

    /**
     * Berechnet das zweidimensionale-int-Array nach den neuen Vorgaben.
     */
    public void recalculateArray() {
        // Prüft die Steine in der aktuellen Iteration
        for (int i = 0; i < this.board.length; ++i) {
            for (int j = 0; j < this.board[i].length; ++j) {
                int nachbarschaft = getNeighborhood(i, j);
                if (nachbarschaft == 3 || (nachbarschaft == 2 && this.board[i][j].isAlive())) {
                    this.board[i][j].aging();
                } else {
                    this.board[i][j].die();
                }
            }
        }

        // Speichert die Änderungen der aktuellen Iteration für die Folgende.
        for (Stone[] row: this.board) {
            for (Stone cell: row) {
                cell.nextLevel();
            }
        }
    }

    /**
     * Berechnet die Nachbarschaft der durch zeile und spalte definierte Zelle (Wenn Nachbarn > 0 wird dieser gezählt)
     * @param zeile zeile der zu berechnenden Zelle
     * @param spalte spalte der zu berechnenden Zelle
     * @return berechnete Nachbarschaft nach den neuen Vorgaben.
     */
    private int getNeighborhood(int zeile, int spalte) {
        int nachbarschaft = 0;

        for (int i = -1; i < 2; ++i) {
            for (int j = -1; j < 2; ++j) {
                if ((i != 0 || j != 0) && (this.board[(zeile + i + this.board.length) % this.board.length][(spalte + j + this.board[zeile].length) % this.board[zeile].length].isAlive())) {
                    ++nachbarschaft;
                }
            }
        }
        return nachbarschaft;
    }

    /**
     * Gibt das übergebene zweidimensionale-boolean-Array zeilenweise auf StOut aus.
     */
    public void printArray() {
        // Array ausgeben
        for (Stone[] zeile : this.board) { // Die Objekte von ausgabe[][] sind zeile[]
            for (Stone zelle : zeile) { // Die Objekte von zeile[] sind zelle
                System.out.print(zelle);
            }
            System.out.println();
        }
    }
}

/**
 * Repräsentiert ein Stein im game of life.
 */
class Stone {
    private int age;
    private int nextage;

    /**
     * Erstellt ein neues Stone-Objekt.
     * @param age Alter des Stones.
     */
    public Stone(int age) { this.age = age; }

    /**
     * Gibt das Alter des Stones zurück.
     * @return Alter des Stones.
     */
    public int getAge() { return age; }

    /**
     * Gibt zurück, ob der Stone lebt.
     * @return Ob Stone lebendig ist.
     */
    public boolean isAlive() { return this.age > 0; }

    /**
     * Tötet den Stein.
     */
    public void die() { this.nextage = 0; }

    /**
     * Lässt den Stein um eine Iteration ältern.
     */
    public void aging() { this.nextage = age + 1; }

    /**
     * Bereitet Stein auf die nächste Iteration vor.
     */
    public void nextLevel() { this.age = nextage; }

    /**
     * Gibt das Zeichen, dass das Alter des Stones repräsentiert als Char zurück.
     * @return Zeichen, dass das Alter des Stones repräsentiert.
     */
    @Override
    public String toString() {
        char c = '*';
        switch (this.age) {
            case 0:
                c = ' ';
                break;
            case 1:
                c = '.';
                break;
            case 2:
                c = 'o';
                break;
            case 3:
                c = 'O';
                break;
        }
        return String.valueOf(c);
    }
}

/**
 * <h1>Übung 11, Aufgabe 01</h1>
 *
 * <p>Programmieren 1
 * Wintersemester 2021/-22
 * HS-Bremerhaven
 * Prof. Dr. Peter Kelb</p>
 *
 * <p>Machen Sie Ihr <i>Life</i> Programm noch „objektorientierter“.</p>
 * <p>Schreiben Sie dazu eine Klasse <i>Stone</i>, die eine Zelle in Ihrem <i>Life</i> Programm
 * darstellt.</p>
 * <p>D.h. das <i>Life</i> Programm speichert nicht mehr ein int-Array sondern ein Stone
 * Array.</p>
 * <p>Statten Sie Ihre <i>Stone</i> Klasse mit notwendigen Methoden aus, wie</p>
 * <ul><li><i>print:</i> Ausgabe</li>
 * <li><i>countNeighbour:</i> zur Berechnung der Anzahl der Nachbarn</li>
 * <li><i>isAlive:</i> ist die Zelle besetzt</li>
 * <li><i>computeNext:</i> wird in der nächsten Iteration die Zelle bestezt sein</li></ul>
 *
 * @author franjogiessel
 */
public class Aufgabe01 {
    public static void main(String[] args) {
        Scanner sc = new Scanner(System.in); // neues Scanner-Objekt sc

        System.out.print("Anzahl Zeilen: ");
        int zeilen = sc.nextInt();
        System.out.print("Anzahl Spalten: ");
        int spalten = sc.nextInt();

        Life life = new Life(zeilen, spalten);
        life.printArray();

        for (int i = 1; i <= 5; ++i) {
            System.out.println("------------------");
            System.out.println("Nach " + i + " Tag" + (i == 1 ? "" : "en" )+ ":");
            life.recalculateArray();
            life.printArray();
        }

    }
}

