package Uebung11;

class Bruch {
    private int zaehler;
    private int nenner;

    /**
     * Erstellt ein Objekt der Klasse Bruch.
     * @param zaehler Zähler-Wert des Bruchs.
     * @param nenner Nenner-Wert des Bruchs.
     */
    public Bruch (int zaehler, int nenner) {
        this.zaehler = nenner < 0 ? -zaehler : zaehler;
        this.nenner = nenner < 0 ? -nenner : nenner;

        kuerzen();
    }

    /**
     * Vergleicht zwei Brüche miteinander. Wenn Bruch1 größer ist, als Bruch2, wird 1 zurückgegeben.
     * Wenn Bruch 2 größer ist, als Bruch 1, wird -1 zurückgegeben.
     * Wenn beide Brüche gleichgroß sind, wird 0 zurückgegeben.
     * @param bruch1 Ausgangsbruch, mit dem verglichen werden soll.
     * @param bruch2 Bruch, der mit dem Ausgangsbruch vergleichen wird.
     * @return Ergebnis der Vergleichs.
     */
    public static int vergleiche(final Bruch bruch1, final Bruch bruch2) {
        int verhaeltnisBruch1 = bruch1.zaehler * bruch2.nenner;
        int verhaeltnisBruch2 = bruch2.zaehler * bruch1.nenner;

        if (verhaeltnisBruch1 > verhaeltnisBruch2) {
            return 1;
        } else if (verhaeltnisBruch1 < verhaeltnisBruch2) {
            return -1;
        } else {
            return 0;
        }
    }

    /**
     * Sortiert das übergebene Bruch-Array mit dem selection sort-Algorithmus und gibt das sortierte Array wieder zurück.
     * Das übergebene Array wird dabei nicht verändert.
     * @param brueche Zu sortierendes Array.
     * @return (Neues) sortiertes Array.
     */
    public static Bruch[] selectionSort(Bruch[] brueche) {
        // Kopiere übergebenes Array, damit dies nicht verändert wird.
        Bruch[] a = new Bruch[brueche.length];
        System.arraycopy(brueche, 0, a, 0, brueche.length);

        for(int i = 0; i < a.length; ++i) {
            int k = i;
            for(int j = i+1; j < a.length; ++j) {
                if(Bruch.vergleiche(a[k], a[j]) == 1){ // if(a[k] > a[j]) {
                    k = j;
                }
            }
            swap(a,i,k);
        }
        return a;
    }

    /**
     * Sortiert das übergebene Bruch-Array mit dem insertion sort-Algorithmus und gibt das sortierte Array wieder zurück.
     * Das übergebene Array wird dabei nicht verändert.
     * @param brueche Zu sortierendes Array.
     * @return (Neues) sortiertes Array.
     */
    public static Bruch[] insertionSort(Bruch[] brueche) {
        // Kopiere übergebenes Array, damit dies nicht verändert wird.
        Bruch[] a = new Bruch[brueche.length];
        System.arraycopy(brueche, 0, a, 0, brueche.length);

        for(int i = 0; i < a.length-1; ++i) {
            int k = i+1;
            Bruch tmp = a[i+1];
            while(k > 0 && Bruch.vergleiche(a[k-1], tmp) == 1) {
                a[k] = a[k-1];
                --k;
            }
            a[k] = tmp;
        }
        return a;
    }

    /**
     * Sortiert das übergebene Bruch-Array mit dem bubble sort-Algorithmus und gibt das sortierte Array wieder zurück.
     * Das übergebene Array wird dabei nicht verändert.
     * @param brueche Zu sortierendes Array.
     * @return (Neues) sortiertes Array.
     */
    public static Bruch[] bubbleSort(Bruch[] brueche) {
        // Kopiere übergebenes Array, damit dies nicht verändert wird.
        Bruch[] a = new Bruch[brueche.length];
        System.arraycopy(brueche, 0, a, 0, brueche.length);

        for(int i = 0; i < a.length; ++i) {
            boolean b = false;
            for(int j = 0; j < a.length-(i+1); ++j) {
                if(Bruch.vergleiche(a[j], a[j+1]) == 1) {
                    swap(a,j+1,j);
                    b = true;
                }
            }
            if(!b)
                return a;
        }
        return a;
    }

    /**
     * Tauscht zwei Brüche im Array a mit den Indices i und j.
     * @param a Bruch-Array, in dem die zu tauschenden Brüche enthalten sind.
     * @param i Index des ersten Bruchs im Array.
     * @param j Index des zweiten Bruchs im Array.
     */
    private static void swap(Bruch[] a, int i, int j) {
        Bruch tmp = a[i];
        a[i] = a[j];
        a[j] = tmp;
    }

    /**
     * Berechnet den größten gemeinsamen Teiler (ggT) der Zahlen a und b und gibt diesen wieder zurück.
     * @param a Erste Zahl
     * @param b Zweite Zahl
     * @return größten gemeinsamen Teiler.
     */
    private static int ggt(int a, int b) {
        // Die größere der beiden Zahlen muss als erstes an den Algorithmus übergeben werden.
        int va = a > b ? a : b;
        int vb = a > b ? b : a;
        int r = 1, vr = 1;
        while (r != 0) {
            vr = r;
            r = va % vb;
            va = vb;
            vb = r;
        }
        return vr;
    }

    /**
     * Gibt den Bruch auf der Console aus.
     */
    @Override
    public String toString() {
        return this.zaehler + " / " + this.nenner + "      (" + 1.0 * this.zaehler/this.nenner + ")";
    }

    /**
     * Multipliziert den Bruch mit dem übergebenen Bruch.
     * @param bruch2 zweiter zu multiplizierender Bruch.
     * @return Ergebnis der Multiplikation als Bruch.
     */
    public Bruch multiplikation(Bruch bruch2) {
        return new Bruch(this.zaehler * bruch2.zaehler, this.nenner * bruch2.nenner);
    }

    /**
     * Dividiert den Ausgangsbruch durch den übergebenen Bruch.
     * @param bruch2 Divisor-Bruch.
     * @return Ergebnis der Division als Bruch.
     */
    public Bruch division(Bruch bruch2) {
        return new Bruch(this.zaehler * bruch2.nenner , this.nenner * bruch2.zaehler);
    }

    /**
     * Addiert den Bruch mit dem übergebenen Bruch.
     * @param bruch2 zweiter zu addierender Bruch.
     * @return Ergebnis der Addition als Bruch.
     */
    public Bruch addition(Bruch bruch2) {
        return new Bruch (this.zaehler * bruch2.nenner + bruch2.zaehler * this.nenner , this.nenner * bruch2.nenner);
    }

    /**
     * Subtrahiert den übergebenen Bruch von dem Ausgangsbruch.
     * @param bruch2 zu subtrahierender Bruch.
     * @return Ergebnis der Subtraktion als Bruch.
     */
    public Bruch subtraktion(Bruch bruch2) {
        return new Bruch (this.zaehler * bruch2.nenner - bruch2.zaehler * this.nenner , this.nenner * bruch2.nenner);
    }

    /**
     * Kürzt den Bruch.
     */
    private void kuerzen() {
        int ggt0 = ggt(Math.abs(zaehler), Math.abs(nenner));
        this.zaehler = zaehler / ggt0;
        this.nenner = nenner / ggt0;
    }
}

/**
 * <h1>Übung 11, Aufgabe 02</h1>
 *
 * <p>Programmieren 1
 * Wintersemester 2021/-22
 * HS-Bremerhaven
 * Prof. Dr. Peter Kelb</p>
 *
 * <p>Implementieren Sie die drei Sortiermethoden Insertion-, Selection- und
 * Bubblesort derart, dass sie Arrays von <i>Rational</i> Objekten sortieren.</p>
 *
 * @author franjogiessel
 */
public class Aufgabe02 {
    public static void main(String[] args) {
        Bruch[] array = {new Bruch(2, 1), new Bruch (-4, 6), new Bruch (15, 24),
                new Bruch (3, 8), new Bruch(2, -10), new Bruch(1, 1)};
        for (int i = 0; i < array.length; ++i) {
            System.out.println("Ausgangsbruch Nr. " + i + ":  " + array[i]);
        }

        System.out.println("-------------------------------------------");

        Bruch[] bubbleSort = Bruch.bubbleSort(array);
        for (int i = 0; i < bubbleSort.length; ++i) {
            System.out.println("Bubble Sort Bruch Nr. " + i + ":  " + bubbleSort[i]);
                    }

        System.out.println("-------------------------------------------");

        Bruch[] insertionSort = Bruch.insertionSort(array);
        for (int i = 0; i < insertionSort.length; ++i) {
            System.out.println("Insertion Sort Bruch Nr. " + i + ":  " + insertionSort[i]);
        }

        System.out.println("-------------------------------------------");

        Bruch[] selectionSort = Bruch.selectionSort(array);
        for (int i = 0; i < selectionSort.length; ++i) {
            System.out.println("Selection Sort Bruch Nr. " + i + ":  " + selectionSort[i]);
        }

    }
}