package Uebung12;

abstract class Number {
    /**
     * Sortiert das übergebene Number-Array mit dem selection sort-Algorithmus und gibt das sortierte Array wieder zurück.
     * Das übergebene Array wird dabei nicht verändert.
     * @param numbers Zu sortierendes Array.
     * @return (Neues) sortiertes Array.
     */
    public static Number[] selectionSort(Number[] numbers) {
        // Kopiere übergebenes Array, damit dies nicht verändert wird.
        Number[] a = new Number[numbers.length];
        System.arraycopy(numbers, 0, a, 0, numbers.length);

        for (int i = 0; i < a.length; ++i) {
            int k = i;
            for (int j = i+1; j < a.length; ++j) {
                if (a[j].isLess(a[k])) { // if(a[k] > a[j]) {
                    k = j;
                }
            }
            swap(a,i,k);
        }
        return a;
    }

    /**
     * Sortiert das übergebene Number-Array mit dem insertion sort-Algorithmus und gibt das sortierte Array wieder zurück.
     * Das übergebene Array wird dabei nicht verändert.
     * @param numbers Zu sortierendes Array.
     * @return (Neues) sortiertes Array.
     */
    public static Number[] insertionSort(Number[] numbers) {
        // Kopiere übergebenes Array, damit dies nicht verändert wird.
        Number[] a = new Number[numbers.length];
        System.arraycopy(numbers, 0, a, 0, numbers.length);

        for (int i = 0; i < a.length-1; ++i) {
            int k = i+1;
            Number tmp = a[i+1];
            while (k > 0 && tmp.isLess(a[k-1])) {
                a[k] = a[k-1];
                --k;
            }
            a[k] = tmp;
        }
        return a;
    }

    /**
     * Sortiert das übergebene Number-Array mit dem bubble sort-Algorithmus und gibt das sortierte Array wieder zurück.
     * Das übergebene Array wird dabei nicht verändert.
     * @param numbers Zu sortierendes Array.
     * @return (Neues) sortiertes Array.
     */
    public static Number[] bubbleSort(Number[] numbers) {
        // Kopiere übergebenes Array, damit dies nicht verändert wird.
        Number[] a = new Number[numbers.length];
        System.arraycopy(numbers, 0, a, 0, numbers.length);

        for (int i = 0; i < a.length; ++i) {
            boolean b = false;
            for (int j = 0; j < a.length-(i+1); ++j) {
                if (a[j+1].isLess(a[j])) {
                    swap(a,j+1,j);
                    b = true;
                }
            }
            if (!b)
                return a;
        }
        return a;
    }

    /**
     * Tauscht zwei Numbers im Array a mit den Indices i und j.
     * @param a Number-Array, in dem die zu tauschenden Numbers enthalten sind.
     * @param i Index der ersten Number im Array.
     * @param j Index der zweiten Number im Array.
     */
    private static void swap(Number[] a, int i, int j) {
        Number tmp = a[i];
        a[i] = a[j];
        a[j] = tmp;
    }

    /**
     * Vergleicht die Zahl mit einer anderen Zahl, ob die Ausgangszahl kleiner ist.
     * @return Aktuelle Zahl kleiner als übergebene Zahl.
     */
    abstract public boolean isLess (final Number otherNumber);
}

class Rational extends Number {
    private int zaehler;
    private int nenner;

    /**
     * Erstellt ein Objekt der Klasse Rational.
     * @param zaehler Zähler-Wert des Bruchs.
     * @param nenner Nenner-Wert des Bruchs.
     */
    public Rational(int zaehler, int nenner) {
        this.zaehler = nenner < 0 ? -zaehler : zaehler;
        this.nenner = nenner < 0 ? -nenner : nenner;

        kuerzen();
    }

    /**
     * Berechnet den größten gemeinsamen Teiler (ggT) der Zahlen a und b und gibt diesen wieder zurück.
     * @param a Erste Zahl
     * @param b Zweite Zahl
     * @return größten gemeinsamen Teiler.
     */
    private static int ggt(int a, int b) {
        // Die größere der beiden Zahlen muss als erstes an den Algorithmus übergeben werden.
        int va = a > b ? a : b;
        int vb = a > b ? b : a;
        int r = 1, vr = 1;
        while (r != 0) {
            vr = r;
            r = va % vb;
            va = vb;
            vb = r;
        }
        return vr;
    }

    /**
     * Gibt den Bruch als String zurück.
     */
    @Override
    public String toString() {
        return this.zaehler + " / " + this.nenner + "      (" + 1.0 * this.zaehler/this.nenner + ")";
    }

    /**
     * Multipliziert den Bruch mit dem übergebenen Bruch.
     * @param rational2 zweiter zu multiplizierender Bruch.
     * @return Ergebnis der Multiplikation als Bruch.
     */
    public Rational multiplikation(Rational rational2) {
        return new Rational(this.zaehler * rational2.zaehler, this.nenner * rational2.nenner);
    }

    /**
     * Dividiert den Ausgangsbruch durch den übergebenen Bruch.
     * @param rational2 Divisor-Bruch.
     * @return Ergebnis der Division als Bruch.
     */
    public Rational division(Rational rational2) {
        return new Rational(this.zaehler * rational2.nenner , this.nenner * rational2.zaehler);
    }

    /**
     * Addiert den Bruch mit dem übergebenen Bruch.
     * @param rational2 zweiter zu addierender Bruch.
     * @return Ergebnis der Addition als Bruch.
     */
    public Rational addition(Rational rational2) {
        return new Rational(this.zaehler * rational2.nenner + rational2.zaehler * this.nenner , this.nenner * rational2.nenner);
    }

    /**
     * Subtrahiert den übergebenen Bruch von dem Ausgangsbruch.
     * @param rational2 zu subtrahierender Bruch.
     * @return Ergebnis der Subtraktion als Bruch.
     */
    public Rational subtraktion(Rational rational2) {
        return new Rational(this.zaehler * rational2.nenner - rational2.zaehler * this.nenner , this.nenner * rational2.nenner);
    }

    /**
     * Kürzt den Bruch.
     */
    private void kuerzen() {
        int ggt0 = ggt(Math.abs(zaehler), Math.abs(nenner));
        this.zaehler = zaehler / ggt0;
        this.nenner = nenner / ggt0;
    }

    @Override
    public boolean isLess(Number otherNumber) {
        Rational r = (Rational) otherNumber;
        return zaehler * r.nenner < r.zaehler * nenner;
    }
}

/**
 * <h1>Übung 12, Aufgabe 02</h1>
 *
 * <p>Programmieren 1
 * Wintersemester 2021/-22
 * HS-Bremerhaven
 * Prof. Dr. Peter Kelb</p>
 *
 * <p>Definieren Sie eine abstrakte Klasse <i>Number</i> und leiten von dieser Klasse Ihre
 * Klasse <i>Rational</i> ab. Stellen Sie Ihre Sortierverfahren um, so dass diese ein Array
 * von <i>Number</i> Objekten erwartet. Welche abstrakten Methodendeklarationen
 * braucht die abstrakte Klasse <i>Number</i>?</p>
 *
 * @author franjogiessel
 */
public class Aufgabe02 {
    public static void main(String[] args) {
        Rational[] array = {new Rational(2, 1), new Rational(-4, 6), new Rational(15, 24),
                new Rational(3, 8), new Rational(2, -10), new Rational(1, 1)};
        for (int i = 0; i < array.length; ++i) {
            System.out.println("Ausgangsbruch Nr. " + i + ":  " + array[i]);
        }

        System.out.println("-------------------------------------------");

        Number[] bubbleSort = Rational.bubbleSort(array);
        for (int i = 0; i < bubbleSort.length; ++i) {
            System.out.println("Bubble Sort Bruch Nr. " + i + ":  " + bubbleSort[i]);
                    }

        System.out.println("-------------------------------------------");

        Number[] insertionSort = Rational.insertionSort(array);
        for (int i = 0; i < insertionSort.length; ++i) {
            System.out.println("Insertion Sort Bruch Nr. " + i + ":  " + insertionSort[i]);
        }

        System.out.println("-------------------------------------------");

        Number[] selectionSort = Rational.selectionSort(array);
        for (int i = 0; i < selectionSort.length; ++i) {
            System.out.println("Selection Sort Bruch Nr. " + i + ":  " + selectionSort[i]);
        }
    }
}